/*
*********************************************************************************************************
*
*                                     MICRIUM BOARD SUPPORT PACKAGE
*
*                          (c) Copyright 2003-2007; Micrium, Inc.; Weston, FL
*
*               All rights reserved.  Protected by international copyright laws.
*
*               Knowledge of the source code may NOT be used to develop a similar product.
*
*               Please help us continue to provide the Embedded community with the finest
*               software available.  Your honesty is greatly appreciated.
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*
*                                BOARD SUPPORT PACKAGE (BSP) FUNCTIONS
*
*                                           CSB ARM MAIN FILE
*
* Filename      : bsp.h
* Version       : V1.89
* Programmer(s) : Jean-Denis Hatier
*********************************************************************************************************
*/

#ifndef  __BSP_H__
#define  __BSP_H__


/*
*********************************************************************************************************
*                                                DEFINES
*********************************************************************************************************
*/

                                                                /* Clock definitions.                                   */
#define  BSP_CLK_AT91RM9200_SLOW_HZ                    32768    /* Slow clock (fixed external 32KHz crystal).           */
#define  BSP_CLK_AT91RM9200_MAIN_HZ                  3686400    /* Main clock (fixed external 3.6864MHz crystal).       */
                                                                /* Master clock (80MHz).                                */
#define  BSP_CLK_AT91RM9200_MASTER_HZ           (BSP_CLK_AT91RM9200_CPU_HZ / 2)
                                                                /* Processor clock (160MHz).                            */
#define  BSP_CLK_AT91RM9200_CPU_HZ              (BSP_CLK_AT91RM9200_MAIN_HZ * 87 / 2)

                                                                /* AT91RM9200 debug port baud rate                      */
#define  BSP_DBGU_RATE                                 38400


/*
*********************************************************************************************************
*                                          FUNCTION PROTOTYPES
*********************************************************************************************************
*/

void         BSP_Init            (void);


/*
*********************************************************************************************************
*                                             TICK SERVICES
*********************************************************************************************************
*/

void         Tmr_Init            (void);


/*
*********************************************************************************************************
*                                           OS/VIEW SERVICES
*********************************************************************************************************
*/

void         OSView_TmrInit      (void);                        /* Initialize the free running timer.                   */
CPU_INT32U   OSView_TmrRd        (void);                        /* Read       the free running timer.                   */


/*
*********************************************************************************************************
*                                             LED SERVICES
*********************************************************************************************************
*/

void         LED_Init            (void);
void         LED_On              (CPU_INT08U  led);
void         LED_Off             (CPU_INT08U  led);
void         LED_Toggle          (CPU_INT08U  led);


/*
*********************************************************************************************************
*                                         PUSH BUTTON SERVICES
*********************************************************************************************************
*/

CPU_BOOLEAN  PB_GetStatus        (CPU_INT08U  push_button);


/*
*********************************************************************************************************
*                                           SERIAL SERVICES
*********************************************************************************************************
*/

void         Ser_RdStr           (CPU_CHAR   *string,
                                  CPU_INT32U  len);


/*
*********************************************************************************************************
*                                              MODULE END
*********************************************************************************************************
*/

#endif  /* __BSP_H__ */
