/*
*********************************************************************************************************
*
*                                             EXAMPLE CODE
*
*                          (c) Copyright 2003-2007; Micrium, Inc.; Weston, FL
*
*               All rights reserved.  Protected by international copyright laws.
*
*               Knowledge of the source code may NOT be used to develop a similar product.
*
*               Please help us continue to provide the Embedded community with the finest
*               software available.  Your honesty is greatly appreciated.
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*
*                                             EXAMPLE CODE
*
* Filename      : app.c
* Version       : V1.91
* Programmer(s) : JDH
*                 ITJ
*********************************************************************************************************
*/

/*
*********************************************************************************************************
*                                             INCLUDE FILES
*********************************************************************************************************
*/

#include  <includes.h>

#if APP_TCPIP_EN
#include  <net.h>
#endif

#if APP_TTCP_EN
#include  <ttcp.h>
#endif


/*
*********************************************************************************************************
*                                               VARIABLES
*********************************************************************************************************
*/

static  OS_STK       AppStartTaskStk[APP_OS_CFG_START_TASK_STK_SIZE];
static  OS_STK       App1_TaskStk[APP_OS_CFG_1_TASK_STK_SIZE];

#if APP_TCPIP_EN
static  NET_IP_ADDR  App_IP_Addr;
static  NET_IP_ADDR  App_IP_Mask;
static  NET_IP_ADDR  App_IP_DfltGateway;
static  NET_IP_ADDR  App_IP_DNS_Srvr;
static  NET_IP_ADDR  App_IP_NTP_Srvr;
#endif

static  CPU_INT32S   App_Clk_UTC_Offset;

static  CPU_INT08S   OS_CPU_UsageMax;


/*
*********************************************************************************************************
*                                          FUNCTION PROTOTYPES
*********************************************************************************************************
*/

static  void  App_TaskCreate(void);
static  void  App_TaskStart (void  *p_arg);
static  void  App_Task_1    (void  *p_arg);

#if APP_TCPIP_EN
static  void  App_InitTCPIP (void);
#endif



/*
*********************************************************************************************************
*                                             C ENTRY POINT
*********************************************************************************************************
*/

int  main (void)
{
#if (OS_TASK_NAME_SIZE >= 6)
    INT8U  os_err;
#endif


   (void)&App_Clk_UTC_Offset;

    os_err = 0;                                                 /* Warning: With some debuggers the first call is       */
                                                                /* ignored.                                             */

    BSP_Init();                                                 /* Initialize BSP.                                      */
    CPU_Init();                                                 /* Initialize CPU.                                      */

    APP_TRACE_INFO(("\n\n\n\r"));
    APP_TRACE_INFO(("Initialize OS ...\n\r"));
    OSInit();                                                   /* Initialize OS.                                       */

                                                                /* Create start task.                                   */
    os_err = OSTaskCreateExt((void (*)(void *)) App_TaskStart,
                             (void          * ) 0,
                             (OS_STK        * )&AppStartTaskStk[APP_OS_CFG_START_TASK_STK_SIZE - 1],
                             (INT8U           ) APP_OS_CFG_START_TASK_PRIO,
                             (INT16U          ) APP_OS_CFG_START_TASK_PRIO,
                             (OS_STK        * )&AppStartTaskStk[0],
                             (INT32U          ) APP_OS_CFG_START_TASK_STK_SIZE,
                             (void          * ) 0,
                             (INT16U          )(OS_TASK_OPT_STK_CLR | OS_TASK_OPT_STK_CHK));

                                                                /* Give a name to tasks.                                */
#if (OS_TASK_NAME_SIZE >= 6)
    OSTaskNameSet(OS_TASK_IDLE_PRIO,          "Idle",  &os_err);
#if (OS_TASK_STAT_EN > 0)
    OSTaskNameSet(OS_TASK_STAT_PRIO,          "Stat",  &os_err);
#endif
    OSTaskNameSet(APP_OS_CFG_START_TASK_PRIO, "Start", &os_err);
#endif

    APP_TRACE_INFO(("Start OS ...\n\r"));
    OSStart();                                                  /* Start OS.                                            */
}


/*
*********************************************************************************************************
*                                           App_TaskStart()
*
* Description : Startup task example code
*
* Arguments   : p_arg       Argument passed by 'OSTaskCreate'.
*
* Returns     : none.
*
* Note(s)     : (1) As mentioned in the book's text, you MUST initialize the ticker only once multitasking
*                   has started.
*
*               (2) The first line of code is used to prevent a compiler warning because 'p_arg' is not
*                   used.  The compiler should not generate any code for this statement.
*********************************************************************************************************
*/

static  void  App_TaskStart (void  *p_arg)
{
#if (CPU_CFG_NAME_EN == DEF_ENABLED)
    CPU_ERR   cpu_err;
    CPU_CHAR  cpu_host_name[CPU_CFG_NAME_SIZE];
#endif


   (void)&p_arg;                                                /* Prevent compiler warning.                            */

                                                                /* ------------- INIT APPS / TASKS / BSP -------------- */
    APP_TRACE_INFO(("Initialize OS timer...\n\r"));
    Tmr_Init();                                                 /* Initialize OS timer.                                 */

#if (CPU_CFG_NAME_EN == DEF_ENABLED)                            /* Initialize CPU host name.                            */
    Str_Copy((CPU_CHAR *)&cpu_host_name[0],
             (CPU_CHAR *)"CSB637");
    APP_TRACE_INFO(("Initialize CPU host name ... "));
    CPU_NameSet((CPU_CHAR *)&cpu_host_name[0],
                (CPU_ERR  *)&cpu_err);
    if (cpu_err == CPU_ERR_NONE) {
        APP_TRACE_INFO(("%s\n\r", &cpu_host_name[0]));
    } else {
        APP_TRACE_INFO(("failed!\n\r"));
    }
#endif

#if (OS_TASK_STAT_EN > 0)
    APP_TRACE_INFO(("Initialize OS statistic task ...\n\r"));
    OSStatInit();                                               /* Initialize OS statistic task.                        */
#endif


    APP_TRACE_INFO(("Create application tasks ...\n\r"));
    App_TaskCreate();                                           /* Create application task.                             */

#if APP_TCPIP_EN
    App_InitTCPIP();                                            /* Initialize TCP/IP stack.                             */
#endif

#if APP_TTCP_EN                                                 /* TTCP contains active user input code. It cannot work */
                                                                /* with other code because it takes all the CPU.        */
    APP_TRACE_INFO(("\n\r"));
    APP_TRACE_INFO(("****************************************************************************\n\r"));
    APP_TRACE_INFO(("*                                                                          *\n\r"));
    APP_TRACE_INFO(("*              Micrium uC/TCP-IP TTCP Performance measurement              *\n\r"));
    APP_TRACE_INFO(("*                     AT91RM9200 on Cogent CSB637 SDK                      *\n\r"));
    APP_TRACE_INFO(("*                                                                          *\n\r"));
    APP_TRACE_INFO(("****************************************************************************\n\r"));
    APP_TRACE_INFO(("\n\r"));
    TTCP_Init();                                                /* Initialize TTCP application                          */
#else
    APP_TRACE_INFO(("\n\r"));
    APP_TRACE_INFO(("System ready.\n\r"));
#endif

    LED_Off(1);
    LED_Off(2);
    LED_Off(3);

    while (DEF_YES) {                                           /* Task body, always written as an infinite loop.       */
        OSTimeDlyHMSM(0, 0, 0, 500);
    }
}


#if APP_TCPIP_EN
/*
*********************************************************************************************************
*                                        INITIALIZE TCP/IP STACK
*********************************************************************************************************
*/

static  void  App_InitTCPIP (void)
{
    NET_ERR      err;
    CPU_INT08U  *pmac;


    APP_TRACE_DEBUG(("Initialize TCP/IP stack...\n"));

    pmac = (CPU_INT08U *)mon_getenv("ETHERADD");
    NetASCII_Str_to_MAC(pmac, NetIF_MAC_Addr, &err);

    err = Net_Init();
    if (err != NET_ERR_NONE) {
        APP_TRACE_DEBUG(("Net_Init() failed: error #%d, line #%d.\n", err, __LINE__));
        while (DEF_YES) {
            ;
        }
    }


    App_IP_Addr        = NetASCII_Str_to_IP(APP_CFG_IP_ADDR_STR_THIS_HOST,    &err);
    App_IP_Mask        = NetASCII_Str_to_IP(APP_CFG_IP_ADDR_STR_NET_MASK,     &err);
    App_IP_DfltGateway = NetASCII_Str_to_IP(APP_CFG_IP_ADDR_STR_DFLT_GATEWAY, &err);
    App_IP_DNS_Srvr    = NetASCII_Str_to_IP(APP_CFG_IP_ADDR_STR_DNS_SRVR,     &err);
    App_IP_NTP_Srvr    = NetASCII_Str_to_IP(APP_CFG_IP_ADDR_STR_NTP_SRVR,     &err);
    App_Clk_UTC_Offset = APP_CFG_CLK_UTC_OFFSET;

    NetIP_CfgAddrThisHost(App_IP_Addr, App_IP_Mask);
    NetIP_CfgAddrDfltGateway(App_IP_DfltGateway);

    APP_TRACE_DEBUG(("    MAC address = %s\n", pmac));
    APP_TRACE_DEBUG(("    IP address  = %s\n", APP_CFG_IP_ADDR_STR_THIS_HOST));
    APP_TRACE_DEBUG(("    IP mask     = %s\n", APP_CFG_IP_ADDR_STR_NET_MASK));
    APP_TRACE_DEBUG(("    IP gateway  = %s\n", APP_CFG_IP_ADDR_STR_DFLT_GATEWAY));

   (void)&App_IP_DNS_Srvr;
   (void)&App_IP_NTP_Srvr;
}
#endif


/*
*********************************************************************************************************
*                                       CREATE APPLICATION TASKS
*
* Description : This function creates the application tasks.
*
* Arguments   : None.
*********************************************************************************************************
*/

static  void  App_TaskCreate (void)
{
#if (OS_TASK_NAME_SIZE >= 9)
    INT8U  os_err;
#endif


    os_err = OSTaskCreateExt((void (*)(void *)) App_Task_1,
                             (void          * ) 0,
                             (OS_STK        * )&App1_TaskStk[APP_OS_CFG_1_TASK_STK_SIZE - 1],
                             (INT8U           ) APP_OS_CFG_1_TASK_PRIO,
                             (INT16U          ) APP_OS_CFG_1_TASK_PRIO,
                             (OS_STK        * )&App1_TaskStk[0],
                             (INT32U          ) APP_OS_CFG_1_TASK_STK_SIZE,
                             (void          * ) 0,
                             (INT16U          )(OS_TASK_OPT_STK_CLR | OS_TASK_OPT_STK_CHK));

#if (OS_TASK_NAME_SIZE >= 9)
    OSTaskNameSet(APP_OS_CFG_1_TASK_PRIO, "LED task", &os_err);
#endif
}


/*
*********************************************************************************************************
*                                                TASK #1
*
* Description : This is an example of a task.
*
* Arguments   : p_arg       argument passed to 'AppTask1()' by 'OSTaskCreate()'.
*
* Notes       : 1) The first line of code is used to prevent a compiler warning because 'p_arg' is not
*                  used.  The compiler should not generate any code for this statement.
*
*               2) Interrupts are enabled once the task start because the I-bit of the CCR register was
*                  set to 0 by 'OSTaskCreate()'.
*
*               3) This Task uses the LEDs and the switch of the CSB937 board,
*                  it blinks user LED 2 on the CSB937.
*********************************************************************************************************
*/

static  void  App_Task_1 (void  *p_arg)
{
   (void)&p_arg;                                                /* Prevent compiler warning.                            */


    OS_CPU_UsageMax = 0;
    while (DEF_ON) {                                            /* Task body, always written as an infinite loop.       */
        LED_Toggle(1);
        OSTimeDlyHMSM(0, 0, 0, 250);
        if (OS_CPU_UsageMax < OSCPUUsage) {
            OS_CPU_UsageMax = OSCPUUsage;
        }
    }
}


/*
*********************************************************************************************************
*********************************************************************************************************
**                                         uC/OS-II APP HOOKS
*********************************************************************************************************
*********************************************************************************************************
*/

#if (OS_APP_HOOKS_EN > 0)
/*
*********************************************************************************************************
*                                      TASK CREATION HOOK (APPLICATION)
*
* Description : This function is called when a task is created.
*
* Argument(s) : ptcb   is a pointer to the task control block of the task being created.
*
* Note(s)     : (1) Interrupts are disabled during this call.
*********************************************************************************************************
*/

void  App_TaskCreateHook (OS_TCB *ptcb)
{
}

/*
*********************************************************************************************************
*                                    TASK DELETION HOOK (APPLICATION)
*
* Description : This function is called when a task is deleted.
*
* Argument(s) : ptcb   is a pointer to the task control block of the task being deleted.
*
* Note(s)     : (1) Interrupts are disabled during this call.
*********************************************************************************************************
*/

void  App_TaskDelHook (OS_TCB *ptcb)
{
   (void)&ptcb;
}


/*
*********************************************************************************************************
*                                      IDLE TASK HOOK (APPLICATION)
*
* Description : This function is called by OSTaskIdleHook(), which is called by the idle task.  This hook
*               has been added to allow you to do such things as STOP the CPU to conserve power.
*
* Argument(s) : none.
*
* Note(s)     : (1) Interrupts are enabled during this call.
*********************************************************************************************************
*/

void  App_TaskIdleHook (void)
{
}


/*
*********************************************************************************************************
*                                        STATISTIC TASK HOOK (APPLICATION)
*
* Description : This function is called by OSTaskStatHook(), which is called every second by uC/OS-II's
*               statistics task.  This allows your application to add functionality to the statistics task.
*
* Argument(s) : none.
*********************************************************************************************************
*/

void  App_TaskStatHook (void)
{
}


/*
*********************************************************************************************************
*                                        TASK SWITCH HOOK (APPLICATION)
*
* Description : This function is called when a task switch is performed.  This allows you to perform other
*               operations during a context switch.
*
* Argument(s) : none.
*
* Note(s)     : (1) Interrupts are disabled during this call.
*
*               (2) It is assumed that the global pointer 'OSTCBHighRdy' points to the TCB of the task that
*                   will be 'switched in' (i.e. the highest priority task) and, 'OSTCBCur' points to the
*                  task being switched out (i.e. the preempted task).
*********************************************************************************************************
*/

#if (OS_TASK_SW_HOOK_EN > 0)
void  App_TaskSwHook (void)
{
}
#endif


/*
*********************************************************************************************************
*                                     OS_TCBInit() HOOK (APPLICATION)
*
* Description : This function is called by OSTCBInitHook(), which is called by OS_TCBInit() after setting
*               up most of the TCB.
*
* Argument(s) : ptcb    is a pointer to the TCB of the task being created.
*
* Note(s)     : (1) Interrupts may or may not be ENABLED during this call.
*********************************************************************************************************
*/

void  App_TCBInitHook (OS_TCB *ptcb)
{
   (void)&ptcb;
}


/*
*********************************************************************************************************
*                                        TICK HOOK (APPLICATION)
*
* Description : This function is called every tick.
*
* Argument(s) : none.
*
* Note(s)     : (1) Interrupts may or may not be ENABLED during this call.
*********************************************************************************************************
*/

#if (OS_TIME_TICK_HOOK_EN > 0)
void  App_TimeTickHook (void)
{
}
#endif


#endif

